/*
 * File rpc.h
 *
 * This file defines the message types that are used in the protocol,
 * the types that are used for the arguments, and the three functions
 * that can be called.
 */

/* Argument Types */
#define ARG_TYPE_SHIFT       (16)

#define ARG_CHAR             (1 << ARG_TYPE_SHIFT)
#define ARG_SHORT            (2 << ARG_TYPE_SHIFT)
#define ARG_INT              (3 << ARG_TYPE_SHIFT)
#define ARG_LONG             (4 << ARG_TYPE_SHIFT)
#define ARG_FLOAT            (5 << ARG_TYPE_SHIFT)
#define ARG_DOUBLE           (6 << ARG_TYPE_SHIFT)
#define ARG_STRING           (7 << ARG_TYPE_SHIFT)

#define ARG_OUTPUT	     (1 << 30)
#define ARG_INPUT	     (1 << 31)

#define ARG_ARRAY_LEN_MASK   (0x0000FFFF)
#define ARG_TYPE_MASK        (0x00FF0000)

#define ARG_MAX_STRING_LEN   (255)

/* Messages */

#define MSG_REGISTER         (1)
#define MSG_REGISTER_SUCCESS (2)
#define MSG_REGISTER_FAILURE (3)
#define MSG_INFO_REQUEST     (4)
#define MSG_INFO_REPLY       (5)
#define MSG_EXECUTE          (6)
#define MSG_EXECUTE_SUCCESS  (7)
#define MSG_EXECUTE_FAILURE  (8)
#define MSG_TERMINATE        (9)

/* Simple definitions that work for our purpose */
/* Note: this is not good practice because many processors
 * can be used in big or little endian modes.
 */
#ifdef sparc
#define BIG_ENDIAN
#else
#ifdef mips
#define LITTLE_ENDIAN
#else
add_definitions_here;
#endif /* mips */
#endif /* sparc */



/******************************************************
 * Implement the following functions 
 ******************************************************/

/* Client Stub Functions */

int rpcCall(char *name, int *argTypes, void **args);
int rpcTerminate(void);


/* Server Skeleton Functions */
typedef int (*function)(int *, void **);

int rpcRegister(char *name, int *argTypes, function f);
int rpcExecute(void);

